//Author: Matthias Walter (matthias.walter@listrik.de)

/* --------------------------- EEPROM --------------------------- 
 Actual Read/Write with:
		eeprom_write_...
		eeprom_read_...
	
	Examples:
		char TempStr[17];
		eeprom_write_block((const void*)TempStr, (void*)EEregelzeit, sizeof(TempStr));
	
		uint16_t MyInt16;
		eeprom_read_block((void*)MyInt16, (const void*)EESum_as_laden, sizeof(EESum_as_laden));
	--------------------------- EEPROM --------------------------- 
	
Static =	in general section: invisible to other Programs
					in Function or Procedure: value will be preserved after leaving that section
Volatile = Compiler not allowed to buffer variable in a register
Volatile static = combination of both
	 
Logical Operators:
	if ((a==4)&&(b==3)) ==> true if both () are true
	if 0b01111110 & 0b00100001 ==> 0b00100000				

Use VOLATILE to force compiler to keep variable, even if unused.
 
 -------------------------------------------------------------------------
Nextion commands:   (9600 Baud, no parity, 1 stop)
ser.write( "page 0") ==> select page 0 Format : ASCII code / 0xff / 0xff / 0xff
ser.write("ref t0") ==> refresh component t0  Format: ASCII code/0xff/0xff/0xff

ser.write( "j0.val=123") ==> write value to component j0 Format : ASCII code / 0xff / 0xff / 0xff
ser.write("t0.txt="Test"") ==> write Text to component t0 Format: ASCII code/0xff/0xff/0xff

ser.write("tsw b0,1") ==> enable component b0 touch event. Format: ASCII code/0xff/0xff/0xff
ser.write("tsw b0,0") ==> enable component b0 release event. Format: ASCII code/0xff/0xff/0xff

Touch event format 0x65/0x00/0x02/0x01/0xff/0xff/0xff  ==> Page 0, Button 2, Pressed
|    |    |    |    |Finished
|    |    |    |Pressed
|    |    |Button 2
|    |page 0
|ID

ser.write("get t0.txt") ==> get t0 text". Returns = "Hallo" Format: 0x70/ASCII code/0xff/0xff/0xff
|    |     |Data
|    |Command
|ID
ser.write("get j0.val") ==> get j0 value. Returns = 123     Format: 0x71/4 byte binary/0xff/0xff/0xff little endian
|    |     |Data
|    |Command
|ID
-------------------------------------------------------------------------
C-Code PI-Controller example, see:
http://www.mikrocontroller.net/topic/82685

See also
http://eleccelerator.com/fusecalc/fusecalc.php?chip=atmega644p
http://www.mikrocontroller.net/articles/AVR_Fuses
http://www.engbedded.com/fusecalc/
G:\users\mwalter\documents\Pedelec\Target\Fuse-Reset.odt      

Windows/Atmel Data Format Difference:
  Example (int32_t)97,8017,389 
		(Windows) Big-Endian stores the number as   : 3A 4B 5C 6D		(Little Endian)
		(Atmel)		Little-Endian stores the number as: 6D 5C 4B 3A		(Big Endian)
		
	Example (int16_t)12.500	 
	 (Windows) Little-Endian stores the number as: 30 D4
	 (Atmel)   Big-Endian stores the number as   : D4 30				
	 
	 
 -------------------------- EEPROM --------------------------
 For actual read/write use 
	eeprom_write_...
	eeprom_read_...
	
	All data types:
		char TempStr[17];
		eeprom_write_block((const void*)TempStr, (void*)EEregelzeit, sizeof(TempStr));
	
		uint16_t MyInt16;
		eeprom_read_block((void*)MyInt16, (const void*)EESumme_as_laden, sizeof(EESumme_as_laden));

 Data is stored in this order to EEPROM !!!				*/
 
#include <avr/io.h>

//G:\Users\mwalter\Documents\Listrik01\Berechnungen.xlsm

/* 0=LED1 OFF
	 1=LED1 in Commutate_ISR
	 2=LED1 in Pedelec_ISR
	 3=LED1 in Timer0_OFV        
	 4=LED1 in Main Loop						
	 5=LED1 in normal 1Hz cycle. 10Hz flash if U-Battery is too low
	 6=LED1 in Timer3_OFV 
	 7=LED1 in Do_Task()			
	 8=LED1 in SendAndGetMessage	
	 9=LED1 in Start_Reset button		 			*/
#define LEDflash						0

#define Target							1																								//0=Pedelec,   1=Listrik01
#define NumHall							3																								//Hall sensors 3 or 4
#define DisplayType					0																								//0=Android OR Display3000, 1=Nextion, 2=internal analog keyboard
#define PowerBlockTest			0
#define Speedsimulation			0
#define FanControl					0																								//0 = Use Fan.  1 = Use Lights
#define PrefixMode					0																								//0 =  1-Byte Prefix. 1 = 4-Byte Prefix
#define SystemVoltage				24																							//Battery Voltage 24, 36, 48V
#define RxBuffMax						60
#define ButtonPullUp        true																						//Debounce buttons TRUE: pull up to 5V,  FALSE: pull down to Ground

enum TargetDirections {
	TargetDORFwd = 0,
	TargetDORRev = 1,
	TargetDORIdl = 2,
	TargetDORNone = 3
};
enum TargetDirections TargetDOR	= TargetDORIdl;
enum TargetDirections OldTargetDOR = TargetDORNone;
	
//>>>>>>>>>>> Aus irgendeinem Grund ist der Wert einer const nicht wie definiert, deshalb alle in var umgewandelt <<<<<<<<<<<<<<

#if Target == 0											//Pedelec only
	#define UseAusgabeTicks		1																								//0=Use Wheel pulse, 1=UseAusgabeTicks
	
	volatile float 
		PedalPulsesPerRev				= 4,
		WheelPulsesPerRev				= 1;
#endif

#if Target == 1											//Listrik0 only
	volatile int16_t 
		Acc_Anz									= 50;																						//500mS averaging time
		
	volatile uint16_t 
		MaxTimeNotHorizontal		= 700;																					//700 * 0,01 = 7 Sec
				
	volatile int32_t 
		MaxNeigVorwaerts				= 250,																					//in 0,1 Grad
		MaxNeigRueckwaerts			= -250,																					//in 0,1 Grad
		MaxNeigBremsVorwaerts		= 200,																					//in 0,1 Grad
		MaxNeigBremsRueckwaerts	= -200,																					//in 0,1 Grad
		NeigNeigMax							=	280;																					//in 0,1 Grad
		
	volatile float
		NickNormalization				= 3.41333;	
#endif

volatile uint16_t 
	#if SystemVoltage == 24																										//24V, 12V pro Akku * 2 Akku
		Akkuleer								= 1920,
		Akkufastleer						= 2300;
	#endif

	#if SystemVoltage == 36																										//36V, 12V pro Akku * 3 Akku
		Akkuleer								= 3000,
		Akkufastleer						= 3500;
	#endif

	#if SystemVoltage == 48																										//48V, 12V pro Akku * 4 Akku
		Akkuleer								= 4000,
		Akkufastleer						= 4700;
	#endif
//>>>>>>>>>>> Aus irgendeinem Grund ist der Wert einer const nicht wie definiert, deshalb alle in var umgewandelt <<<<<<<<<<<<<<

//----------------------- EEPROM ----------------------- 
uint8_t
	EDummy									EEMEM;																						//Do not use first Byte

uint16_t
	EMaxV										EEMEM,
	EMinV										EEMEM,
	ETotalKilometer					EEMEM,
	EKilometer							EEMEM,
	EMeter									EEMEM,
	ECentimeter							EEMEM;
	
uint32_t																																		//int32_t in the program is stored as uint32_t in EEPROM
	ESum_as_charge					EEMEM,
	ESum_as_discharge				EEMEM,
	ESum_mAh_charge					EEMEM,
	ESum_mAh_discharge			EEMEM,
	EBatterycap							EEMEM,
	ENick_Offset            EEMEM;

float
	EStartKilometer					EEMEM;

char
	EERegelzeit[17]					EEMEM,
	EEMit_Acc_Nick_max[17]	EEMEM,
	EEMit_Acc_Nick[17]			EEMEM,
	EEMit_Acc_Nick_min[17]	EEMEM,
	EEPWM_Total[17]					EEMEM,
	EEUBatt[17]							EEMEM,
	EEAusgabeTicks[17]			EEMEM,
	EECOM_errMain[17]				EEMEM,
	EEUgrund[17]						EEMEM,
	EEPWM_Max[17]						EEMEM,
	EEugemessen[17]					EEMEM,
	EEImin[17]							EEMEM,
	EEImax[17]							EEMEM,
	EESum_mah_discharge[17] EEMEM,
	EESum_mah_charge[17]		EEMEM,
	EEVerbraucht[17]				EEMEM,
	EEBatterycap[17]				EEMEM,
	EEKilometer[17]					EEMEM,
	EEMeter[17]							EEMEM,
	EECentimeter[17]				EEMEM,
	EERegelzeitMax[17]			EEMEM,
	EEnull_nick[17]					EEMEM,
	EEP_Factor[17]					EEMEM,
	EEI_Factor[17]					EEMEM,
	EED_Factor[17]					EEMEM,
	EEMP_Factor[17]					EEMEM,
	EEMV_Factor[17]					EEMEM,
	EEAusgabeTicksMax[17]		EEMEM,
	EEMaxV[17]							EEMEM,
	EEMinV[17]							EEMEM;
//----------------------- EEPROM ----------------------- 

//--------- vars in interrupt routine that are written to, must be VOLATILE -----------

volatile bool																
	Frei								= false,
	EStop								= false,
	Sensor_OK						= false,
	DoTask							= false,
  MotorOFF						= true,
	PWMisON							= false,
	dirup								= true,
	Go									= false;
	
volatile uint8_t	
	FootCount						= 0,
	Bstate							= 0,
	Motorposition				= 0,
	Aposition						= 0,
	PWM_Max							= 0,																									//Grter PWM-Wert whrend der Fahrt
	PWMmax							= 200,																								//Default Limit
	ToutCtr							= 0,
	Position[13],
	BatteryPercent			= 0,																									//0..100%
	BCC									= 0,
	FwdAngle						= 5,
	RevAngle						= 2,
	p										= 0,
	Sensor_OK_Count			= 0,
	ControlStrategy			= 0,																									//0=P,  1=PI,  2=PID
	LTest								= 0,
	InteruptCount				= 0,
	CtrmS								= 0,
	A[16],																																		//0...15
	S[13],																																		//0...12
	DOR[0xff],																																//0...254 ==> 0x00...0xfe
	Sinus[13],																																//0...12
	Wcount							= 0,
	W_count							= 5,
	Ta,
	Tb,
	Tc,
	RxBuffIdx,
	OldTargetPWM				= 255,																								//undefined
	Tx_ID								= 0x65,
	NN,
	nn,
	Minp,
	BatteryChargeState	= 0,
	OldPCICR						= 0,
	OldPCICR1						= 0,
	FanSwitchLevel			= 0,
	PWM_Total						= 0,
	Lights							= 0,
	
	// Defaults. Will be overwritten by Display Controller
	// See Excel table "Sinusbewertung.xls" and "Rotorposition Array A().xls"
	IBattMax						= 30,																									//A default
	
	/*	Bit 0 ==> 1 = Hall unplugged
			Bit 1 ==> 1 = Overcurrent
			Bit 2 ==> 1 = Msg Receive Timeout (not used in Main Control because Main Control also sees timeout)
			Bit 3 ==> 1 = 
			Bit 4 ==> 1 =
			Bit 5 ==> 1 =
			Bit 6 ==> 1 =
			Bit 7 ==> 1 =    		*/
	EStop_Reason				= 0,
	
	/* ==> Message to Display
						PrefixMode=0   PrefixMode=1
							 p=1         p=4
	Tx[0]						0           0..3        Prefix
	Tx[p+0]					1						4						Msg_ID  (0x65..0xFF) 
	Tx[p+1..p+9]		2..10				5..13				Data 1..9
	Tx[p+10]				11					14					BCC
	Tx[p+11] 				12					15					0x0D (Ende-Kennung)
	
	  ==>  Message from Display
											PrefixMode=0   PrefixMode=1
											 p=1               p=4
	Rx[0]										0									0										not used
	Rx[1..p]								1									1..4								Prefix
	Rx[p+1]									2									5										Msg_ID  (0x65..0xFF)
	Rx[p+2..p+RxBuffZeiger] 3..RxBuffZeiger-2	6..RxBuffZeiger-2]	Data1..n
	Rx[RxBuffZeiger-1]      RxBuffZeiger-1		RxBuffZeiger-1			BCC
	Rx[RxBuffZeiger]				Rx[RxBuffZeiger]	Rx[RxBuffZeiger]		0x0D  (Ende-Kennung)		*/	
	#if PrefixMode == 0
		Tx[13],
		Rx[RxBuffMax];																													// **** Rx Buffer **** not "volatile" because memmove doesn't support that		
	#else	
		Tx[16],	
		Rx[RxBuffMax];																													// **** Rx Buffer **** not "volatile" because memmove doesn't support that	
	#endif
	
volatile int16_t
	P_Anteil,
	I_Anteil,
	D_Anteil,
	Ticks								= 0,
	Avg_Acc_Nick_Max		= 0,
	Avg_Acc_Nick_Min		= 0,
	Timer0Int						= 0,
	T2_10ms_PresetVal		= 45536,
	Ibatt_target,
	Max_V,
	Min_V,
	VmaxV								=	220,																								//in 0.1Kmh steps
	VmaxR,
	WriteTicks,
	TicksMax,
	Kmh									= 0,
	Centimeter					= 0,
	Meter								= 0,
	Kilometer						= 0,
	IMotNom,
	Acc_Nick,
	Imax,
	Imin,
	Idummy,
	Null_Nick,																																//Normierter Nick-Offset
	IBatt;																																		//in 0,1A steps
	
int16_t
	Heatsink_Temperature,
	Motor_Temperature;
	
uint16_t
	PWM_Prescaler				= 8,
	ControlTime					= 0,
	ControlTimeMax			= 0,
	
	#if Target == 0
		Timer0_Frequency	= 0,
		WheelCtr					= 0,
		WheelCtr1					= 0,
		PedalCtr					= 0,
		PedalCtr1					= 0,
	#endif
	
	Startup							= 0,
	/*Startup.0  = Start/Reset-Taste entlastet
	  Startup.1  = Kopplung zum Motorcontroller ist OK
	  Startup.2  = Futaster entlastet
	  Startup.3  = Antrieb ist AUS
	  Startup.4  = Systemspannung in Toleranz
	  Startup.5  = PID-Parameter in Toleranz
	  Startup.6  = Sensoren in Mittellage
	  Startup.7  = Futaster belastet
	  Startup.8  = Start/Reset-Taste bettigt
	  Startup.9  = Bitte aufsteigen
	  Startup.10 = ---
	  Startup.11 = ---
	  Startup.12 = ---
	  Startup.13 = ---
	  Startup.14 = ---
	  Startup.15 = ---					*/

	Ioffset							= 512,																							//ZERO current = 512 Counts ==> 2,5V is default,
	Totalkm							= 0,
	Loop_count					= 0,
	UBatt								= 0,																								//10 mV steps
	EstopReason					= 0,
	/*EstopReason.0 = Fusschalter zu lange entlastet
	  EstopReason.1 = zu lange nicht waagerecht
	  EstopReason.2 = zu weit nach vorne gekippt
	  EstopReason.3 = zu weit nach hinten gekippt
	  EstopReason.4 = Hallstecker fehlt          (kommt vom Motor)
	  EstopReason.5 = Motor berstrom            (kommt vom Motor)
	  EstopReason.6 = ---
	  EstopReason.7 = ---			*/

	ErrorWord						= 0,
	/*ErrorWord.0  = Spannung ist < 24V
	  ErrorWord.1  = Spannung ist < 36V
	  ErrorWord.2  = Spannung ist < 48V
	  ErrorWord.3  = Gyrowerte verrauscht
	  ErrorWord.4  = Fuschalter zu lange entlastet
	  ErrorWord.5  = ---
	  ErrorWord.6  = Antrieb ist auf NOT-AUS
	  ErrorWord.7  = Rx Timeout Motor-/Maincontroller
	  ErrorWord.8  = Hallstecker fehlt
	  ErrorWord.9  = ---
	  ErrorWord.10 = Motor berstrom
	  ErrorWord.11 = ---
	  ErrorWord.12 = Unfallschreiber
	  ErrorWord.13 = Betriebsschreiber
	  ErrorWord.14 = ---
	  ErrorWord.15 = ---				*/
	Wdummy							= 0,
	Wd									= 0,
	I_kum								= 0,
	Heatsink_Temp				= 0,
	Motor_Temp					= 0,
	Fancount						= 0,
	COMErrMain					= 0;
	  						
volatile uint32_t
	ADC_Nick						= 0,
	Usum								= 0,
	Rxprefixlen,
	Txprefixlen,
	Nick_Offset;

int32_t
		Brake;
		
volatile int32_t
	//PWM Wert ab dem automatisch gebremst wird um nicht an die PWM-Grenze zu kommen 
	SpwmV								= 190,
	SpwmR								= -190,
	Integrator					= 0,
	Sum_As_discharge		= 0,
	Sum_As_charge				= 0,
	Sum_mAh_discharge		= 0,
	Sum_mAh_charge			= 0,
	BatteryConsumed			= 0,																									//Verbrauchte Ladung in mA-Stunden
	Batterycap					= 0,																									//Battery capacity in mAh
	SettlingTime				= 0,		
	LastabhBrems				= 0,
	GeschwabhBrems			= 0,																									//Bremse bei berlast oder zu hoher Geschw.
	Generated,																																//Erzeugte Ladung in mA-Stunden
	Isum								= 0,
	ChargedPower,																															//mAh
	DischargedPower,	
	Keybd,
	IBattTarget_sum,
	Sum_Acc_Nick				= 0,
	Avg_Acc_Nick				= 0,
	Avg_Acc_Nick_Old		= 0,
	O_Nick_u,																																	//Neigungsgrenze rckwrts
	O_Nick_o,																																	//Neigungsgrenze vorwrts
	O_Nick_oo;																																//Maximale Neigungsgrenze vorwrts		

volatile float 
	//Speed simulation
	#if Target == 1
		SpeedInc					= 3.0,	
		SpeedDec					= -2.5,	
		FwdMax						= 50.0,
		RevMax						= -30.0,
	#endif
	ControlVal					= 0,
	Mv_anpassung				= 0,
	Mp_anpassung				= 0,
	MaxSum_As_discharge	= 360000.0,																						//Amperesekunden
	MaxSum_As_charge		= 360000.0,																						//Amperesekunden
	WheelCircumferance	= 1.96,																								//Meter
	KmhFactor						= 12.7174,
	SmoothingFactor			= 0.75,
	VoltageFactor				=	5.426408,																						//0.01V Resolution
	CurrentFactor				= 6.826666,																						//mA
	Efficiency					= 0.85,
	TempFactorLM35			= 0.146484,																						//0,1Cels resolution	
	PedalRPM,
	Pedalfactor,
	WheelSpeed,
	Icontrol_Pfactor,
	Icontrol_Ifactor,	
	StartKilometer,
	K_TotalAdj,
	Kv,
	P_Factor						= 0.0033,
	I_Factor						= 0.00006,
	D_Factor						= 0.08,
	MotPwr_Factor				= 0.006,																							//Load dependent
	MotVel_Factor				= 0.0;																								//Geschwindigkeitsabhngig;

char
	Ugrund[33];																																//Auslsegrund. 32 character + Null
	
void BetriebsDatenSchreiber(void);
void UnfallDatenSchreiber(bool erase);
uint8_t SendAndGetMessage(bool Send_Only);
void GetMotorPosition(void);
void Commutate(void);
void Do_Task(void);
void GetADCdata(void);
int16_t ReadNeigung(void);
#if Target == 0
	void Pedelec_ISR(void);
#endif	

//Debounce
uint8_t get_key_state(uint8_t key_mask);
uint8_t get_key_press(uint8_t key_mask);
uint8_t get_key_short(uint8_t key_mask); 
uint8_t get_key_long(uint8_t key_mask);
uint8_t get_key_repeated(uint8_t key_mask); 

/* PWM_Start:
  >>>>>>>>>>>>>>>>>>>>> ATMEGA168 <<<<<<<<<<<<<<<<<<
	 TCCR0A = &b00000000
				      00|||||| ==> OC0A nicht mit Timer / Counter verbunden
						  01|||||| ==> OC0A nicht mit Timer/Counter verbunden wenn WGM13=0 (W)aveform (G)eneration (M)ode
						  10|||||| ==> Clear OC0A on Compare Match when counting up, set OC0A on Compare Match when counting down
					    11|||||| ==> Set OC0A on Compare Match when counting up, clear OC0A on Compare Match when counting down
								00|||| ==> OC0B nicht mit Timer/Counter verbunden
								01|||| ==> OC0B nicht mit Timer/Counter verbunden wenn WGM13=0 (W)aveform (G)eneration (M)ode
								10|||| ==> Clear OC0B on Compare Match when counting up, set OC0B on Compare Match when counting down
								11|||| ==> Set OC0B on Compare Match when counting up, clear OC0B on Compare Match when counting down
									0000 Mode 0 ==> Normal Modus Timer zhlt von 0 bis 65535 luft dann ber und beginnt erneut bei 0. Der 
									||||            normale Modus ist in alle Timer/Counter integriert. Das TCNTx Register wird nicht gelscht, 
									||||						bei einem berlauf startet das Register von 0. Bei aktiviertem Timer/Counter Overflow 
									||||            Interrupt wird dieser ausgelst. Man kann den Wert, bei dem ein Interrupt ausgelst werden 
									||||            soll, auch durch das Vergleichsregister OCRx einstellen. Dafr aktiviert man den Compare Match 
									||||            Interrupt, der ausgelst wird, sobald TCNTx = OCRx ist. In diesem Modus kann man einen Timer 
									||||            Overflow Interrupt periodisch auslsen. Zustzlich kann bei einem bestimmten Wert des 
									||||						Zhlregisters ein Output Compare Match Interrupt ausgelst werden.
									0001 Mode 1 ==> PWM Phase-Correct 8 Bit
									||||            (8 Bit ==> bis 256 zhlen)
									0010 Mode 2 ==> PWM Phase-Correct 9 Bit - Es wird von 0 bis zur Obergrenze und wieder zurck gezhlt.
									||||            (9 Bit ==> bis 512 zhlen)
									0011 Mode 3 ==> PWM Phase-Correct 10 Bit
									||||            (10 Bit ==> bis 1024 zhlen)
									0100 Mode 4 ==> CTC Modus (Clear Timer on Compare Match) Timer zhlt von Null bis zum  Vergleichswert, der im
									||||            Vergleichsegister (Compare1A) abgelegt werden muss, ndert dann den Status von OCR1A und fngt
									||||            wieder bei Null an.
									||||						
									||||						Fast-PWM Modus: 
									||||							Alle Timer/Counter sind mit dem Fast PWM Mode ausgestattet. Der Counter zhlt immer vom 
									||||							minimalen aufwrts zum maximalen Wert und beginnt wieder beim minimalen Wert. Dies ermglicht
									||||							einen schnellen Funktionsgenerator. Der Ausgang wird beim Starten auf 1 gesetzt und beim 
									||||							maximalen Wert auf 0.
									0101 Mode 5 ==> Fast-PWM-Mode 8 Bit zhlt Timer vom Vergleichswert A aufwrts bis Vergleichswert B,
									||||						fngt dann wieder bei A an.
									0110 Mode 6 ==> Fast-PWM-Mode 9 Bit zhlt Timer vom Vergleichswert A aufwrts bis Vergleichswert B,
									||||            fngt dann wieder bei A an
									0111 Mode 7 ==> Fast-PWM-Mode 10 Bit zhlt Timer vom Vergleichswert A aufwrts bis Vergleichswert B,
									||||            fngt dann wieder bei A an
									1000 Mode 8 ==> Fast-PWM-Mode zhlt Timer vom Vergleichswert A aufwrts bis Vergleichswert B, fngt dann 
									||||            wieder bei A an
									||||
									||||						Phase_Correct Mode: Alle Timer/Counter beherschen diesen Modus.
									||||							Der Phasenkorrekte PWM Modus ermglicht genaue Funktionsgeneratoren. Hier wird erst hoch 
									||||							gezhlt und danach wieder herunter, wobei der Ausgang OC beim Hochzhlen ahlen auf 0 und 
									||||							beim Herunterzhlen auf 1 gesetzt ist. Dass das Vergleichsregister OCR1A (bzw. ICR1) den 
									||||							maximalen Zhlwert bestimmt, kann so Pulsfrequenzmodulation implementiert werden.
									1001 Mode 9 ==> Phase Correct-PWM Mode. Timer zhlt von 0 aufwrts bis zum Vergleichswert dann wieder
									||||            zurck bis 0 (Ausgang OCR1A)
									1010 Mode 10 ==> Phase Correct-PWM Mode
									1011 Mode 11 ==> Phase Correct-PWM Mode (Ausgang OCR1A)
									1100 Mode 12 ==> CTC- Modus = (C)lear (T)imer on (C)ompare
									||||             Alle Timer/Counter des ATMega32  besitzen diesen Modus. Stimmt das TCNTx Register mit dem
									||||             Vergleichsregister OCRx berein, wird das TCNTx Register auf 0 gesetzt und bei aktiviertem 
									||||             Timer/Counter Overflow Interrupt wird dieser ausgelst. Zum aktivieren des CTC-Modus mssen 
									||||             die WGM-Bits (W)aveform (G)eneration (M)ode, im TCCRx Register entsprechend gesetzt sein. 
									||||             Dieser Modus ermglicht es periodisch einen Timer Overflow Interrupt auszulsen, wobei die 
									||||             Zeit zwischen 2 Interrupts verndert werden kann
									1101 Mode 13 ==> Reserved
									1110 Mode 14 ==> Fast-PWM-Mode (ICR1 )
									1111 Mode 15 ==> Fast-PWM-Mode (Ausgang OCR1A)
		
	 TCCR0B = &B00000000
							00||0||| ==> Timer Mode
							01||0||| ==> PWM Phase Correct
							00||1||| ==> CTC Mode. Siehe oben
			  		  01||1||| ==> Fast PWM
				        00|||| ==> Timer Pin 0C0 nicht mit Ausgangspin verbunden
				        01|||| ==> Ausgangspin schaltet bei berlauf
				        10|||| ==> Setzt 0C0 auf "0" beim berlauf
				        11|||| ==> Setzt 0C0 auf "1" beim berlauf
					        0||| ==> Reserviert
				           000 ==> Timer steht
									 001 ==> Prescaler = 1
								 	 010 ==> Prescaler = 8
								 	 011 ==> Prescaler = 64
									 100 ==> Prescaler = 256
									 101 ==> Prescaler = 1024

	>>>>>>>>>>>>>>>>>>>>> ATMEGA644/1284/2560 <<<<<<<<<<<<<<<<<<
	 TCCR0A = &b00000000
					    00|||||| ==> OC0A nicht mit Timer/Counter verbunden
					    01|||||| ==> WGM02=0: Normal operation. OC0A disconnected
								|||||| ==> WGM02=1: Toggle 0C0A on compare match
					    10|||||| ==> Clear OC0A on compare match. Set 0C0A at bottom
					    11|||||| ==> Set OC0A on compare catch. Clear 0C0A at bottom
					      00|||| ==> OC0B nicht mit Timer/Counter verbunden
								01|||| ==> Reserved
								10|||| ==> Clear OC0B on compare match. Set 0C0B at bottom
								11|||| ==> Set OC0B on compare catch. Clear 0C0B at bottom
									00|| ==> Reserved
										00 Mode 0 ==> Normal Modus Timer zhlt von 0 bis 255 bzw 65535, luft dann ber und beginnt erneut bei 0. 
										||            Der normale Modus ist in alle Timer/Counter integriert. Das TCNTx Register wird nicht gelscht, 
										||						bei einem berlauf startet das Register von 0. Bei aktiviertem Timer/Counter Overflow Interrupt
										||            wird dieser ausgelst. Man kann den Wert, bei dem ein Interrupt ausgelst werden soll, auch 
										||            durch das Vergleichsregister OCRx einstellen. Dafr aktiviert man den Compare Match Interrupt,
										||            der ausgelst sobald TCNTx = OCRx ist. In diesem Modus kann man einen Timer Overflow Interrupt 
										||            periodisch auslsen. Zustzlich k,ann bei einem bestimmten Wert des Zhlregisters ein Output 
										||						Compare Match Interrupt ausgelst werden.
										01 Mode 1 ==> PWM Phase-Correct. Update 0CRx at Top
										10 Mode 2 ==> CTC. Update 0CRx immediate
										11 Mode 3 ==> Fast PWM. Update 0CRx at Bottom
										00 Mode 4 ==> Siehe auch TCCR0B. Reserved
										01 Mode 5 ==> Siehe auch TCCR0B. PWM-Phase correct. . Set 0C0A at top
										10 Mode 6 ==> Siehe auch TCCR0B. Reserved
										11 Mode 7 ==> Siehe auch TCCR0B. Fast PWM. Update 0CRx at Bottom
				
	TCCR0B = &B00000000         >>>Prescaler  Ergebnis muss > 5 KHz sein sonst reicht Kondensator der IR2183 nicht
				     0||0|||| ==> Force output Compare A
							0|||||| ==> Force output Compare B
							 00|||| ==> Reserved
					       1||| ==> WGM02=0 => PWM MOde 0..6  WGM02=1 => PWM MOde 4..7
						      000 ==> Clock stopped
									001 ==> Prescaler = 1
									010 ==> Prescaler = 8
									011 ==> Prescaler = 64
									100 ==> Prescaler = 256
									101 ==> Prescaler = 1024
									110 ==> External clock source on T0. Clock falling edge
									111 ==> External clock source on T0. Clock rising edge
									
									
----------------------------------------------------------------------------------------------
PCICR = (P)in(C)hange(I)nterrupt(C)hange(R)egister

Bit0 ==> PCIE0: Pin Change Interrupt Enable 0 (Port B0..B7)
When the PCIE0 bit is set (one) and the I-bit in the Status Register (SREG) is set (one), pin
Change Interrupt 0 Is Enabled. Any Change On Any Enabled Pcint7 : 0 Pin Will Cause An Interrupt.
The corresponding interrupt of Pin Change Interrupt Request is executed from the PCI0 Interrupt
Vector. Pcint7 : 0 Pins Are Enabled Individually by the PCMSK0 Register.

Bit1 ==> PCIE1: Pin Change Interrupt Enable 1 (Port C0..C7)
When the PCIE1 bit is set (one) and the I-bit in the Status Register (SREG) is set (one), pin
change interrupt 1 is enabled. Any change on any enabled PCINT15:8 pin will cause an  interrupt.
The corresponding interrupt of Pin Change Interrupt Request is executed from the PCI1
Interrupt Vector. PCINT15:8 pins are enabled individually by the PCMSK1 Register.

Bit2 ==> PCIE2: Pin Change Interrupt Enable 2 (Port D0..D7)
When the PCIE2 bit is set (one) and the I-bit in the Status Register (SREG) is set (one), pin
change interrupt 2 is enabled. Any change on any enabled PCINT23:16 pin will cause an interrupt.
The corresponding interrupt of Pin Change Interrupt Request is executed from the PCI2
Interrupt Vector. PCINT23:16 pins are enabled individually by the PCMSK2 Register.

Bit3 ==> PCIE3: Pin Change Interrupt Enable 3
When the PCIE3 bit is set (one) and the I-bit in the Status Register (SREG) is set (one), pin
change interrupt 3 is enabled. Any change on any enabled PCINT31..24 pin will cause an interrupt.
The corresponding interrupt of Pin Change Interrupt Request is executed from the PCI3
Interrupt Vector. Pcint31..24 Pins Are Enabled Individually By The PCMSK3 Register.
*/